"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Document = void 0;
var documents_1 = require("@langchain/core/documents");
Object.defineProperty(exports, "Document", { enumerable: true, get: function () { return documents_1.Document; } });
