/**
 * Prompt for trajectory evaluation chain.
 */
import { ChatPromptTemplate } from "@langchain/core/prompts";
export declare const EVAL_CHAT_PROMPT: ChatPromptTemplate<any, any>;
export declare const TOOL_FREE_EVAL_CHAT_PROMPT: ChatPromptTemplate<any, any>;
