"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Docstore = void 0;
/**
 * Abstract class for a document store. All document stores should extend
 * this class.
 */
class Docstore {
}
exports.Docstore = Docstore;
