"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatMessageHistory = void 0;
var chat_history_1 = require("@langchain/core/chat_history");
Object.defineProperty(exports, "ChatMessageHistory", { enumerable: true, get: function () { return chat_history_1.InMemoryChatMessageHistory; } });
