export declare function logVersion010MigrationWarning({ oldEntrypointName, newEntrypointName, newPackageName, }: {
    oldEntrypointName: string;
    newEntrypointName?: string;
    newPackageName?: string;
}): void;
export declare function logVersion020MigrationWarning({ oldEntrypointName, newEntrypointName, newPackageName, }: {
    oldEntrypointName: string;
    newEntrypointName?: string;
    newPackageName?: string;
}): void;
