"use strict";
/* #__PURE__ */ console.warn([
    `[ERROR]: The "langchain/prompts" entrypoint was removed in LangChain version 0.2.0.`,
    ``,
    `Instead, please import from "@langchain/core/prompts".`,
].join("\n"));
