"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MaskingTransformer = void 0;
/**
 * Abstract class representing a transformer used for masking and rehydrating messages.
 */
class MaskingTransformer {
}
exports.MaskingTransformer = MaskingTransformer;
