"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatToOpenAIToolMessages = void 0;
const tool_calling_js_1 = require("./tool_calling.cjs");
Object.defineProperty(exports, "formatToOpenAIToolMessages", { enumerable: true, get: function () { return tool_calling_js_1.formatToToolMessages; } });
