import { Serializable } from "@langchain/core/load/serializable";
/**
 * Base class for all file stores. All file stores should extend this
 * class.
 */
export class BaseFileStore extends Serializable {
}
