/**
 * Sleep for a given amount of time.
 * @param ms - The number of milliseconds to sleep for. Defaults to 1000.
 * @returns A promise that resolves when the sleep is complete.
 */
export declare function sleep(ms?: number): Promise<void>;
